using System;
using System.Collections.Generic;
using System.Linq;
using System.IO;
using UnityEngine;
using UnityEngine.Video;

namespace FunctionalDevices
{
    public class FunctionalDevices
    {
        public static void Main()
        { 
            CategoryBuilder.Add("Devices", "", "Category");
            AddDevice("Phone", 0.4f);
            AddDevice("Tablet", 0.5f);
            AddDevice("Microwave", 1.5f);
            AddDevice("TV", 3.5f);
            AddDevice("Computer", 2.3f);
            AddDevice("Monitor", 1.2f);
            AddDevice("Gas Cooker", 7.7f);
        }
        public static void AddDevice(string name, float Mass)
        { 
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = name+ " -FDM",
                    NameToOrderByOverride = "!",
                    CategoryOverride = ModAPI.FindCategory("Devices"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/"+name+"Thumb.png"),
                    AfterSpawn = (Instance) =>
                    {
                        GameObject.Destroy(Instance.GetComponent<GlowingHotMetalBehaviour>());

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/"+name+"Off.png");

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Flammable metal");
                        phys.TrueInitialMass = Mass;
                        phys.InitialMass = Mass;
                        phys.rigidbody.mass = Mass;

                        foreach (var colliders in Instance.GetComponents<Collider2D>())
                        {
                            GameObject.Destroy(colliders);
                        }
                        Instance.AddComponent<BoxCollider2D>();

                        if(name == "Phone" || name == "Tablet")
                        {
                            Instance.GetComponent<SpriteRenderer>().sortingOrder = -1;
                            var phoneOrTabletBehaviour = Instance.GetOrAddComponent<PhoneAndTabletBehaviour>();
                            phoneOrTabletBehaviour.BreakingThreshold = 1.5f;
                            phoneOrTabletBehaviour.TemperatureThreshold = 40f;
                            var Sprite1 = new PhoneAndTabletBehaviour.DeviceSprites();
                            Sprite1.OnDevice = ModAPI.LoadSprite("Textures/"+name+"On.png");
                            Sprite1.OffDevice = ModAPI.LoadSprite("Textures/"+name+"Off.png");
                            Sprite1.BrokenDevice = ModAPI.LoadSprite("Textures/"+name+"Broken.png");
                            var Sprite2 = new PhoneAndTabletBehaviour.DeviceSprites();
                            Sprite2.OnDevice = ModAPI.LoadSprite("Textures/"+name+"2On.png");
                            Sprite2.OffDevice = ModAPI.LoadSprite("Textures/"+name+"2Off.png");
                            Sprite2.BrokenDevice = ModAPI.LoadSprite("Textures/"+name+"2Broken.png");
                            phoneOrTabletBehaviour.Textures = new PhoneAndTabletBehaviour.DeviceSprites[] 
                            {
                                Sprite1,
                                Sprite2,
                            };
                        }
                        if(name == "Microwave")
                        {
                            Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                            var background = new GameObject("Background");
                            background.transform.parent = Instance.transform;
                            background.transform.rotation = Instance.transform.rotation;
                            background.transform.localPosition = new Vector3(0f, 0f);
                            background.transform.localScale = new Vector3(1f, 1f);
                            var backgroundSprite = background.GetOrAddComponent<SpriteRenderer>();
                            backgroundSprite.sprite = ModAPI.LoadSprite("Textures/MicrowaveBackground.png");
                            backgroundSprite.sortingLayerName = "Background";
                            var microwaveBehaviour = Instance.GetOrAddComponent<MicrowaveBehaviour>();
                            microwaveBehaviour.Sound1 = ModAPI.LoadSound("Sounds/Microwave1.mp3");
                            microwaveBehaviour.Sound2 = ModAPI.LoadSound("Sounds/Microwave2.mp3");
                            microwaveBehaviour.Sound3 = ModAPI.LoadSound("Sounds/Microwave3.mp3");
                            microwaveBehaviour.OnDevice = ModAPI.LoadSprite("Textures/MicrowaveOn.png");
                            microwaveBehaviour.OffDevice = ModAPI.LoadSprite("Textures/MicrowaveOff.png");
                            microwaveBehaviour.BrokenDevice = ModAPI.LoadSprite("Textures/MicrowaveBroken.png");
                            microwaveBehaviour.BreakingThreshold = 5.5f;
                            microwaveBehaviour.TemperatureThreshold = 50f;
                            microwaveBehaviour.audioSource = Instance.AddComponent<AudioSource>();
                        }
                        if(name == "TV")
                        {
                            var renderer = Instance.GetComponent<SpriteRenderer>();
                            renderer.sortingLayerName = "Foreground";
                            var screen = new GameObject("Screen");
                            screen.transform.parent = Instance.transform;
                            screen.transform.rotation = Instance.transform.rotation;
                            screen.transform.localPosition = new Vector3(0f, 0.1f);
                            screen.transform.localScale = new Vector3(1f, 1f);
                            var screenSprite = screen.GetOrAddComponent<SpriteRenderer>();
                            screenSprite.sprite = ModAPI.LoadSprite("Textures/TVScreen.png");
                            screenSprite.sortingLayerName = renderer.sortingLayerName;
                            screenSprite.sortingOrder = renderer.sortingOrder + 1;
                            var tvBehaviour = Instance.GetOrAddComponent<TVBehaviour>();
                            tvBehaviour.videoPlayer = screen.AddComponent<VideoPlayer>();
                            tvBehaviour.audioSource = Instance.AddComponent<AudioSource>();
                            tvBehaviour.screenRenderer = screenSprite;
                            tvBehaviour.Video = "https://cdn.discordapp.com/attachments/864114883931275294/888109763739672586/TVChannel1.m4v";
                            tvBehaviour.Audio = ModAPI.LoadSound("Sounds/TVChannel1.mp3");
                            tvBehaviour.BrokenDevice = ModAPI.LoadSprite("Textures/TVBroken.png");
                            tvBehaviour.BreakingThreshold = 13f;
                            tvBehaviour.TemperatureThreshold = 40f;
                        }
                        if(name == "Computer")
                        {
                            var renderer = Instance.GetComponent<SpriteRenderer>();
                            renderer.sortingLayerName = "Foreground";
                            var glow = new GameObject("Glow");
                            glow.transform.parent = Instance.transform;
                            glow.transform.rotation = Instance.transform.rotation;
                            glow.transform.localPosition = new Vector3(0f, 0f);
                            glow.transform.localScale = new Vector3(1f, 1f);
                            var glowSprite = glow.GetOrAddComponent<SpriteRenderer>();
                            glowSprite.enabled = false;
                            glowSprite.sprite = ModAPI.LoadSprite("Textures/ComputerGlow.png");
                            glowSprite.sortingOrder = renderer.sortingOrder + 1;
                            glowSprite.sortingLayerName = renderer.sortingLayerName;
                            var computerBehaviour = Instance.GetOrAddComponent<ComputerBehaviour>();
                            computerBehaviour.glowRenderer = glowSprite;
                            computerBehaviour.BrokenDevice = ModAPI.LoadSprite("Textures/ComputerBroken.png");
                            computerBehaviour.BreakingThreshold = 8f;
                            computerBehaviour.TemperatureThreshold = 45f;
                            computerBehaviour.MaxFrame = 8;
                            computerBehaviour.Delay = 25f;                          
                            computerBehaviour.Colors = new Color32[]
                            {
                                new Color32(255, 0, 0, 255),
                                new Color32(255, 180, 0, 255),
                                new Color32(255, 255, 0, 255),
                                new Color32(0, 255, 0, 255),
                                new Color32(0, 255, 255, 255),
                                new Color32(0, 0, 255, 255),
                                new Color32(190, 0, 220, 255),
                                new Color32(225, 120, 220, 255),
                            };
                        }
                        if(name == "Monitor")
                        {
                            Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                            var monitorBehaviour = Instance.GetOrAddComponent<MonitorBehaviour>();
                            monitorBehaviour.OnDevice = ModAPI.LoadSprite("Textures/MonitorOn.png");
                            monitorBehaviour.OffDevice = ModAPI.LoadSprite("Textures/MonitorOff.png");
                            monitorBehaviour.BrokenDevice = ModAPI.LoadSprite("Textures/MonitorBroken.png");
                            monitorBehaviour.BreakingThreshold = 5f;
                            monitorBehaviour.TemperatureThreshold = 40f;
                        }
                        if(name == "Gas Cooker")
                        {
                            Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                            var background = new GameObject("Background");
                            background.transform.parent = Instance.transform;
                            background.transform.rotation = Instance.transform.rotation;
                            background.transform.localPosition = new Vector3(0f, 0f);
                            background.transform.localScale = new Vector3(1f, 1f);
                            var backgroundSprite = background.GetOrAddComponent<SpriteRenderer>();
                            backgroundSprite.sprite = ModAPI.LoadSprite("Textures/Gas CookerBackground.png");
                            backgroundSprite.sortingLayerName = "Background";
                            var gasCookerBehaviour = Instance.GetOrAddComponent<GasCookerBehaviour>();
                            gasCookerBehaviour.OnDevice = ModAPI.LoadSprite("Textures/Gas CookerOn.png");
                            gasCookerBehaviour.OffDevice = ModAPI.LoadSprite("Textures/Gas CookerOff.png");
                            gasCookerBehaviour.BrokenDevice = ModAPI.LoadSprite("Textures/Gas CookerBroken.png");
                            gasCookerBehaviour.BreakingThreshold = 65f;
                            gasCookerBehaviour.TemperatureThreshold = 50f;
                        }
                    }
                }
            );
        }
    }
    public class CategoryBuilder
    {
        public static void Add(string name, string description, string thumbnail)
        {
            CatalogBehaviour manager = UnityEngine.Object.FindObjectOfType<CatalogBehaviour>();
            if (manager.Catalog.Categories.FirstOrDefault((Category c) => c.name == name) == null)
            {
                Category category = ScriptableObject.CreateInstance<Category>();
                category.name = name;
                category.Description = description;
                category.Icon = ModAPI.LoadSprite(thumbnail+".png");
                Category[] NewCategories = new Category[manager.Catalog.Categories.Length + 1];
                Category[] categories = manager.Catalog.Categories;
                for (int i = 0; i < categories.Length; i++)
                {
                    NewCategories[i] = categories[i];
                }
                NewCategories[NewCategories.Length - 1] = category;
                manager.Catalog.Categories = NewCategories;
            }
        }
    }
    public class PhoneAndTabletBehaviour : MonoBehaviour
    {
        private void Start()
        {
            phys = gameObject.GetComponent<PhysicalBehaviour>();
            renderer = gameObject.GetComponent<SpriteRenderer>();
            light = ModAPI.CreateLight(gameObject.transform, Color.white, 1.8f, 0f);
            chosenIndex = UnityEngine.Random.Range(0, Textures.Length - 0);
            UpdateActivation();
        }
        private void Use()
        {
            Activated = !Activated;
            UpdateActivation();
        }
        private void Update()
        {
            if(!Broken && phys.Temperature > TemperatureThreshold)
            {
                UpdateDamage();
            }
        }
        private void UpdateActivation()
        {
            PhoneAndTabletBehaviour.DeviceSprites deviceSprites = Textures[chosenIndex];
            if(Activated && !Broken)
            {
                renderer.sprite = deviceSprites.OnDevice;
                light.Brightness = 1.5f;
            }
            else
            {
                renderer.sprite = deviceSprites.OffDevice;
                light.Brightness = 0f;
            }
            if(Broken)
            {
                renderer.sprite = deviceSprites.BrokenDevice;
                light.Brightness = 0f;
                Activated = false;
            }
        }
        private void OnEMPHit()
        {
            UpdateDamage();
        }
        private void Shot(Shot shot)
        {
            UpdateDamage();
        }
        private void OnFragmentHit()
        {
            UpdateDamage();
        }
        private void OnCollisionEnter2D(Collision2D collision)
        {
            if(collision.GetContact(0).normalImpulse > BreakingThreshold)
            {
                UpdateDamage();
            }
        }
        private void UpdateDamage()
        {
            if(Broken)
            {
                return;
            }
            Broken = true;
            ModAPI.CreateParticleEffect("BrokenElectronicsSpark", gameObject.transform.position);
            UpdateActivation();
        }

        public bool Broken;
        public bool Activated;
        public SpriteRenderer renderer;
        public PhoneAndTabletBehaviour.DeviceSprites[] Textures;
        public PhysicalBehaviour phys;
        public LightSprite light;
        public int chosenIndex;
        public float TemperatureThreshold;
        public float BreakingThreshold;
        public struct DeviceSprites
        {
            public Sprite OnDevice;
            public Sprite OffDevice;
            public Sprite BrokenDevice;
        }
    }
    public class MicrowaveBehaviour : MonoBehaviour
    {
        private void Start()
        {
            phys = gameObject.GetComponent<PhysicalBehaviour>();
            renderer = gameObject.GetComponent<SpriteRenderer>();
            audioSource = gameObject.GetComponent<AudioSource>();
            audioSource.loop = true;
            audioSource.clip = Sound2 as AudioClip;
            audioSource.spread = 0f;
            audioSource.volume = 1f;
            audioSource.minDistance = 10f;
            audioSource.spatialBlend = 1f;
            audioSource.dopplerLevel = 1f;
            audioSource.time = 0f;
            gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();
            light = ModAPI.CreateLight(gameObject.transform, new Color32(230, 20, 0, 255), 3f, 0f);
            UpdateActivation();
        }
        private void Use()
        {
            Activated = !Activated;
            UpdateActivation();
            if(Activated && !Broken)
            {
                audioSource.PlayOneShot(Sound1, 1f);
            }
            else if(!Broken)
            {
                audioSource.PlayOneShot(Sound3, 1f);
            }
        }
        private void Update()
        {
            if(!Broken && phys.Temperature > TemperatureThreshold)
            {
                UpdateDamage();
            }
        }
        private void UpdateActivation()
        {
            if(Activated && !Broken)
            {
                audioSource.Play();
                renderer.sprite = OnDevice;
                light.Brightness = 1.7f;
            }
            else
            {
                audioSource.Stop();
                renderer.sprite = OffDevice;
                light.Brightness = 0f;
            }
            if(Broken)
            {
                audioSource.Stop();
                renderer.sprite = BrokenDevice;
                light.Brightness = 0f;
                Activated = false;
            }
        }
        private void OnEMPHit()
        {
            UpdateDamage();
        }
        private void Shot(Shot shot)
        {
            UpdateDamage();
        }
        private void OnFragmentHit()
        {
            UpdateDamage();
        }
        private void OnCollisionEnter2D(Collision2D collision)
        {
            if(collision.GetContact(0).normalImpulse > BreakingThreshold)
            {
                UpdateDamage();
            }
        }
        private void UpdateDamage()
        {
            if(Broken)
            {
                return;
            }
            Broken = true;
            ModAPI.CreateParticleEffect("BrokenElectronicsSpark", gameObject.transform.position);
            UpdateActivation();
        }

        public bool Broken;
        public bool Activated;
        public SpriteRenderer renderer;
        public PhysicalBehaviour phys;
        public LightSprite light;
        public AudioSource audioSource;
        public AudioClip Sound1;
        public AudioClip Sound2;
        public AudioClip Sound3;
        public Sprite OnDevice;
        public Sprite OffDevice;
        public Sprite BrokenDevice; 
        public float TemperatureThreshold;
        public float BreakingThreshold;
    }
    public class TVBehaviour : MonoBehaviour
    {
        private void Start()
        {
            phys = gameObject.GetComponent<PhysicalBehaviour>();
            renderer = gameObject.GetComponent<SpriteRenderer>();
            audioSource = gameObject.GetComponent<AudioSource>();
            audioSource.loop = true;
            audioSource.clip = Audio as AudioClip;
            audioSource.spread = 0f;
            audioSource.volume = 1f;
            audioSource.minDistance = 10f;
            audioSource.spatialBlend = 1f;
            audioSource.dopplerLevel = 1f;
            audioSource.time = 0f;
            gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();
            videoPlayer.url = Video;
            videoPlayer.frame = 100;
            videoPlayer.isLooping = true;
            videoPlayer.time = 0f;
            light = ModAPI.CreateLight(gameObject.transform, Color.white, 8f, 0f);
            UpdateActivation();     
        }
        private void Use()
        {
            Activated = !Activated;
            UpdateActivation();
        }
        private void Update()
        {
            videoPlayer.playbackSpeed = Time.timeScale;
            if(!Broken && phys.Temperature > TemperatureThreshold)
            {
                UpdateDamage();
            }
        }
        private void UpdateActivation()
        {
            if(Activated && !Broken)
            {
                audioSource.Play();
                videoPlayer.Play();
                screenRenderer.enabled = true;
                light.Brightness = 0.8f;
            }
            else
            {
                audioSource.Stop();
                videoPlayer.Stop();
                screenRenderer.enabled = false;
                light.Brightness = 0f;
            }
            if(Broken)
            {
                audioSource.Stop();
                videoPlayer.Stop();
                screenRenderer.enabled = false;
                renderer.sprite = BrokenDevice;
                light.Brightness = 0f;
                Activated = false;
            }
        }
        private void OnEMPHit()
        {
            UpdateDamage();
        }
        private void Shot(Shot shot)
        {
            UpdateDamage();
        }
        private void OnFragmentHit()
        {
            UpdateDamage();
        }
        private void OnCollisionEnter2D(Collision2D collision)
        {
            if(collision.GetContact(0).normalImpulse > BreakingThreshold)
            {
                UpdateDamage();
            }
        }
        private void UpdateDamage()
        {
            if(Broken)
            {
                return;
            }
            Broken = true;
            ModAPI.CreateParticleEffect("BrokenElectronicsSpark", gameObject.transform.position);
            UpdateActivation();
        }

        public bool Activated;
        public bool Broken;
        public SpriteRenderer renderer;
        public SpriteRenderer screenRenderer;
        public PhysicalBehaviour phys;
        public VideoPlayer videoPlayer;
        public AudioSource audioSource;
        public LightSprite light;
        public string Video;
        public AudioClip Audio;
        public Sprite BrokenDevice;
        public float TemperatureThreshold;
        public float BreakingThreshold;
    }
    public class ComputerBehaviour : MonoBehaviour
    {
        private void Start()
        {
            phys = gameObject.GetComponent<PhysicalBehaviour>();
            renderer = gameObject.GetComponent<SpriteRenderer>();
            light = ModAPI.CreateLight(gameObject.transform, Color.red, 4f, 0f);
        }
        private void Use()
        {
            Activated = !Activated;
        }
        private void Update()
        {
            if(!Broken && phys.Temperature > TemperatureThreshold)
            {
                UpdateDamage();
            }
        }
        private void FixedUpdate()
        {
            if(Activated && !Broken)
            {
                glowRenderer.enabled = true;
                light.Brightness = 1.5f;
                Clock += 1f;
                if(Clock >= Delay)
                {
                    Clock = 0f;
                    if(Frame >= MaxFrame)
                    {
                        Frame = 0;
                    }
                    else
                    {
                        light.Color = (Colors[Frame]);
                        glowRenderer.color = (Colors[Frame]);
                        Frame += 1;
                    }
                }
            }
            else
            {
                glowRenderer.enabled = false;
                light.Brightness = 0f;
            }
            if(Broken)
            {
                glowRenderer.enabled = false;
                renderer.sprite = BrokenDevice;
                light.Brightness = 0f;
                Activated = false;
            }
        }
        private void OnEMPHit()
        {
            UpdateDamage();
        }
        private void Shot(Shot shot)
        {
            UpdateDamage();
        }
        private void OnFragmentHit()
        {
            UpdateDamage();
        }
        private void OnCollisionEnter2D(Collision2D collision)
        {
            if(collision.GetContact(0).normalImpulse > BreakingThreshold)
            {
                UpdateDamage();
            }
        }
        private void UpdateDamage()
        {
            if(Broken)
            {
                return;
            }
            Broken = true;
            ModAPI.CreateParticleEffect("BrokenElectronicsSpark", gameObject.transform.position);
        }

        public bool Activated;
        public bool Broken;
        public SpriteRenderer renderer;
        public SpriteRenderer glowRenderer;
        public PhysicalBehaviour phys;
        public LightSprite light;
        public Sprite BrokenDevice;
        public Color32[] Colors;
        private int Frame = 0;
        public int MaxFrame;
        public float TemperatureThreshold;
        public float BreakingThreshold;
        public float Delay;
        private float Clock = 99999f;
    }
    public class MonitorBehaviour : MonoBehaviour
    {
        private void Start()
        {
            phys = gameObject.GetComponent<PhysicalBehaviour>();
            renderer = gameObject.GetComponent<SpriteRenderer>();
            light = ModAPI.CreateLight(gameObject.transform, Color.blue, 3.5f, 0f);
            UpdateActivation();
        }
        private void Use()
        {
            Activated = !Activated;
            UpdateActivation();
        }
        private void Update()
        {
            if(!Broken && phys.Temperature > TemperatureThreshold)
            {
                UpdateDamage();
            }
        }
        private void UpdateActivation()
        {
            if(Activated && !Broken)
            {
                renderer.sprite = OnDevice;
                light.Brightness = 2f;
            }
            else
            {
                renderer.sprite = OffDevice;
                light.Brightness = 0f;
            }
            if(Broken)
            {
                renderer.sprite = BrokenDevice;
                light.Brightness = 0f;
                Activated = false;
            }
        }
        private void OnEMPHit()
        {
            UpdateDamage();
        }
        private void Shot(Shot shot)
        {
            UpdateDamage();
        }
        private void OnFragmentHit()
        {
            UpdateDamage();
        }
        private void OnCollisionEnter2D(Collision2D collision)
        {
            if(collision.GetContact(0).normalImpulse > BreakingThreshold)
            {
                UpdateDamage();
            }
        }
        private void UpdateDamage()
        {
            if(Broken)
            {
                return;
            }
            Broken = true;
            ModAPI.CreateParticleEffect("BrokenElectronicsSpark", gameObject.transform.position);
            UpdateActivation();
        }

        public bool Broken;
        public bool Activated;
        public SpriteRenderer renderer;
        public PhysicalBehaviour phys;
        public LightSprite light;
        public Sprite OnDevice;
        public Sprite OffDevice;
        public Sprite BrokenDevice;
        public float TemperatureThreshold;
        public float BreakingThreshold;
    }
    public class GasCookerBehaviour : MonoBehaviour
    {
        private void Start()
        {
            phys = gameObject.GetComponent<PhysicalBehaviour>();
            renderer = gameObject.GetComponent<SpriteRenderer>();
            light = ModAPI.CreateLight(gameObject.transform, Color.red, 4.5f, 0f);
            UpdateActivation();
        }
        private void Use()
        {
            Activated = !Activated;
            UpdateActivation();
        }
        private void Update()
        {
            if(!Broken && phys.Temperature > TemperatureThreshold)
            {
                UpdateDamage();
            }
        }
        private void UpdateActivation()
        {
            if(Activated && !Broken)
            {
                renderer.sprite = OnDevice;
                light.Brightness = 2f;
            }
            else
            {
                renderer.sprite = OffDevice;
                light.Brightness = 0f;
            }
            if(Broken)
            {
                renderer.sprite = BrokenDevice;
                light.Brightness = 0f;
                Activated = false;
            }
        }
        private void Shot(Shot shot)
        {
            UpdateDamage();
        }
        private void OnFragmentHit()
        {
            UpdateDamage();
        }
        private void OnCollisionEnter2D(Collision2D collision)
        {
            if(collision.GetContact(0).normalImpulse > BreakingThreshold)
            {
                UpdateDamage();
            }
        }
        private void UpdateDamage()
        {
            if(Broken)
            {
                return;
            }
            Broken = true;
            renderer.sprite = BrokenDevice;
            gameObject.FixColliders();
            GameObject.Destroy(gameObject.transform.Find("Background").gameObject);
            ExplosionCreator.CreateFragmentationExplosion(95U, gameObject.transform.position, 8f, 8f, true, true, 0f);
            UpdateActivation();
        }

        public bool Broken;
        public bool Activated;
        public SpriteRenderer renderer;
        public PhysicalBehaviour phys;
        public LightSprite light;
        public Sprite OnDevice;
        public Sprite OffDevice;
        public Sprite BrokenDevice;
        public float TemperatureThreshold;
        public float BreakingThreshold;
    }
}